.SH
Installing Little Smalltalk
.PP
For most users the following simple steps should suffice to build the
system.
.IP \(bu
Unpack the sources.
.IP \(bu
Edit the file env.h, changing the metadefine to an already known system
type.
.IP \(bu
Type \fImake\fP.
.PP
The remainder of this document is intended to provide more detailed
information for the curious, and to help those individuals for whom the
previous instructions don't work.
.SH
Porting to new systems
.PP
There are many ways in which compilers and operating systems differ 
from each other.
A fair amount of work has been expanded in making sure the software will
operate on most machines, which requires that different code fragements be
used on different systems.  In large part these are controlled by a single
``meta-define'' in the file env.h.  Setting this one value then causes the
expansion of another code segment, which then defines many more options.
.PP
In the event that you are attempting to port the software to a system that
has not previously been defined, you will need to decide which set of
options to enable.  The next two sections contain information you may need
in making this determination.
.SH
Define Options
.PP
Many options are specified merely by giving or not giving a DEFINE
statement in the file env.h.  The following table presents the meaning for
each of these values:
.de Op
.IP \\fB\\$1\\fP
.br
..
.Op BINREADWRITE
Defined if the fopen specification for binary files must include the "b"
modifier.  This is true on many MS-DOS inspired systems.
.Op DELNOTRM
Defined if the ms-dos style DEL command should be used to remove temporary 
files, in place of the unix-style rm command.
.Op NOENUMS
Defined if enumerated datatypes are not supported.  If defined, these will
be replaced by #define constants.
.Op NOTYPEDEF
Defined if the typedef construct is not supported.  If defined, these will
be replaced by #define constructs.
.Op NOVOID
Defined if the void keyword is not recognized.
If defined, expect \fIlint\fP to complain a lot about functions returning
values which are sometimes (or always) ignored.
.Op GAMMA
Used if the name of the function which computes the log-gamma function is
gamma.  This is incompatible with the \fBLGAMMA\fP switch.
If neither gamma() nor lgamma() is available, neither option should be
defined.
.Op LGAMMA
Used if the name of the function which computes the log-gamma function is
lgamma.  This is incompatible with the \fBGAMMA\fP switch.
.Op SIGNALS
Used if \fIboth\fP the <signals.h> package and the <longjmp.h> package are
avilable, to trap interrups from the user.
.Op STRING
Used if the string functions (strcpy, strcat and the like) are found in
<string.h>.  This switch is incompatible with \fBSTRINGS\fP.
.Op STRINGS
Used if the string functions (strcpy, strcat and the like) are found in
<strings.h>.  This switch is incompatible with \fBSTRING\fP.
.LP
In addition, several routines can optionally be replaced by macros for
greater efficiency.  See the file memory.h for more information.
.SH
Object Memory
.PP
There are several datatypes, not directly supported by C, that are used in
the Little Smalltalk system.  The first of these is the datatype byte.
A byte is an eight bit unsigned (hence positive) quantity.
On many systems the appropriate datatype is unsigned char, however on other
systems this declaration is not recognized and other forms may be required.
To aid in coverting to and from bytes the macro byteToInt() is used, which
converts a byte value into an integer.  In addition, the routines byteAt
and byteAtPut are used to get and put bytes from byte strings.
.PP
The other datatype is that used to represent object points.  On most
machines in which a short is 16 bits, the datatype short should suffice.
Much more information on the memory module can be found in the file
memory.h.
.SH
Options in Building the System
.PP
To create the parser type
.DS I
make parse
.DE
.PP
The resulting program, called parse, is used to generate the object image
initially loaded into the bytecode interpreter.
.PP
Next, make the interpreter itself by typing
.DS I
make st
.DE
.PP
Note that the interpreter and the parser share some files.
.PP
Finally, produce an initial object image.  The image created when you type
.DS I
make sunix
.DE
.LP
is the smallest and fastest.  It is a single process version of smalltalk.
A buggy multiprocess version can be created by typing ``make munix''*.
.FS
* Multi processing from munix is done entirely in Smalltalk.
While this is a good idea from the point of view of keeping the bytecode
interpreter small and giving one the greatest flexibility, there seems to
be a dramatic performance penalty.  I'm considering the alternatives.
.FE
Of more interest, an image containing test cases (***currently only
the 8 queens***) can be generated by typing ``make stest''.
In the latter case, the command ``test all'', when given in response to the
prompt (see below), runs all the test cases.
.PP
Once you have created an object image, type 
.DS I
st
.DE
.LP
to run the system.
By default the image file ``imageFile'' is read.  You can optionally
use a different image file by giving the name on the command line following
the st command.
.SH
Compiler Bogosities
.PP
This section will detail some of the unnatural actions you may have to
perform to get Little Smalltalk to work with a few brain damaged compilers.
Since there are so few of these, it I see it as a problem more with the
compilers than with the system, the code make no accomodation for these.
.PP
On some older Sequent Balance C compilers, incorrect code is produced for
.DS I
hash %= (objectSize(symbols) / 2)
.DE
.LP
In the file image.c.  This should be replaced by
.DS I
hash = hash % (objectSize(symbols) / 2)
.DE
.PP
On many systems external names are restricted to six (or even five!)
characters.  This causes significant problems.  I have heard of systems
which automatically generate a sed script to take care of this, but have
not found one yet.  If you know of such a thing let me know.
.SH
Helping Others
.PP
If you succeed in getting Little Smalltalk ported to a machine or operating
system not described in env.h, I would be most pleased to get a listing of
the changes you made.  These can then be incorporated into the latest
distribution.
