*
* Little Smalltalk, version 2
* Written by Tim Budd, Oregon State University, July 1987
*
*  methods for the unix front end - single process version
*
*	(override previous declaration, adding new instance variable)
Declare Smalltalk Object errorRecoveryBlock
*	(better override instance as well )
Instance Smalltalk smalltalk
*
Class Method
	executeWith: arguments
		^ ( Context new ; method: self ; 
			temporaries: ( Array new: temporarySize) ;
			arguments: arguments )
		   executeFrom: 0
]
Class Class
	addMethod
		self doEdit: ''
|
	addSubClass		| name |
		name <- (smalltalk getPrompt: 'Class Name? ') asSymbol.
		globalNames at: name put: 
			( Class new; name: name ; superClass: self ;
				readInstanceVariables; readMethods )
|
	doEdit: startingText		| theMethod text |
		theMethod <- Method new.
		text <- startingText.
		[ text <- text edit. theMethod text: text.
		  (theMethod compileWithClass: self)
			ifTrue: [ methods at: theMethod name put: theMethod .
				  smalltalk flushMessageCache. false ]
			ifFalse: [ smalltalk inquire:
				'edit again (yn) ? ' ] ] whileTrue
|
	display
		('Class name: ', name asString)  print.
		(superClass notNil)
			ifTrue: [ ('Superclass: ', superClass ) print ].
		'Instance Variables:' print.
		variables isNil
			ifTrue: [ 'no instance variables ' print ]
			ifFalse: [ variables display ]
		'Subclasses: ' print.
		self subClasses display
|
	editMethod: name
		self doEdit: ( methods at: name
			ifAbsent: [ 'no such method ' print. ^ nil ] ) text
|
	readInstanceVariables		| index text |
		variables <- List new.
		text <- smalltalk getPrompt: 'Instance Variables? '.
		[ text <- text copyFrom: 
			(text indexOf: [:x | x isAlphabetic] 
				ifAbsent: [ text size + 1 ]) to: text size.
		  text size > 0 ] whileTrue:
			[ index <- text indexOf: [:x | x isAlphabetic not]
					ifAbsent: [ text size + 1].
			  variables addLast: (text copyFrom: 1 to: index - 1).
			  text <- text copyFrom: index to: text size ].
		self variables:  variables asArray
|
	readMethods
		[ smalltalk inquire: 'Add a method (yn) ? ' ]
			whileTrue: [ self addMethod ]
|
	viewMethod: name
		(methods at: name
			ifAbsent: [ 'no such method ' print. ^ nil ]) text print
]
Class Smalltalk
	error: aString
		('Error: ' ,  aString) print.
		errorRecoveryBlock value
|
	getString
		^ <1>
|
	getPrompt: aString
		aString printNoReturn.
		^ self getString
|
	init		| string |
		[ string <- self getPrompt: '>	'. string notNil ]
			whileTrue: [ (string size > 0)
					ifTrue: [ self doIt: string ] ]
|
	inquire: aString	| response |
		response <- self getPrompt: aString.
		response isNil 
			ifTrue: [ ^ false ].
		^ 'yY' includes: (response at: 1 ifAbsent: [])
|
	doIt: aString		| method |
		errorRecoveryBlock <- [ ^ nil ].
		method <- Method new.
		method text: ( 'proceed ', aString ).
		(method compileWithClass: Smalltalk)
			ifTrue: [ method executeWith: #( 1 ) ]
|
	saveImage		| name |
		name <- self getPrompt: 'type image name: '.
		(self saveImage: name)
			ifTrue: [ ('image ', name, ' created') print ]
			ifFalse: [ 'image not created' print ]
]
Class String
	edit
		^ <89 self>
|
	print
		^ <88 self>
|
	printNoReturn
		^ <86 self>
]
